/*****************************************************************************
 *  Rohde&Schwarz DVSG Digital Video Signal Generator instrument driver
 *  VXIpnp Instrument Driver
 *  Original Release: August 2010
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *****************************************************************************/
#include <math.h>
#include <float.h>
#include "rsdvsg.h"
#include "rsdvsg_utility.h"

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

static ViString signalStandardArr[] = {"StanHDMI","StanHDSDI","StanCOMPOSITE","StanCOMPONENT","StanYC","StanSCART","StanVGA","StanTS",VI_NULL}; 
static ViString statusRegisterArr[] = {"COND","EVEN",VI_NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
ViStatus rsdvsg_CheckStatusCallback (ViSession io);
ViStatus rsdvsg_WaitForOPCCallback (ViSession io);
ViStatus _VI_FUNC  rsdvsg_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsdvsg_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsdvsg_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsdvsg_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rsdvsg); 
    checkErr( Rs_SpecificDriverNew (instrSession, "rsdvsg", newOptionString, attrList));
   
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsdvsg);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsdvsg_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsdvsg_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSDVSG_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSDVSG_ATTR_ID_QUERY_RESPONSE,
                                   0, RSDVSG_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSDVSG_VALID_ID_RESPONSE_STRING, strlen(RSDVSG_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rsdvsg_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;

        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    if (error < VI_SUCCESS)
        (void) Rs_Dispose (*newInstrSession);

    return error;
}

/*===========================================================================*/
/* Function: AV Signal Player Mode
/* Purpose:  This function selects the AV Signal Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_AVSignalPlayerMode(
	ViSession	instrSession
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckErr (rsdvsg_SetAttributeViString (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_MODE, NULL));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Status
/* Purpose:  Select the status of the AV Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerStatus(
	ViSession	instrSession,
	ViInt32	command
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_STATUS,
                                               command), 2, "Command");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Status
/* Purpose:  Queries the actual state of the AV Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerStatus(
	ViSession	instrSession,
	ViInt32*	command
)
{
    ViStatus	error = VI_SUCCESS;
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_STATUS,
                                               command), 2, "Command");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Source
/* Purpose:  Selects the file to be loaded into the AV Signal Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerSource(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_PLAYER_SOURCE,
                                               fileName), 2, "File Name");
  Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Source Parameters
/* Purpose:  This function reads AV signal player parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerSourceParameters(
	ViSession	instrSession,
	ViChar	fileDateSize[],
	ViReal64*	originalLoopTime,
	ViChar	originalTSDataRate[]
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
      
	viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_FILE_INFO,
												RSDVSG_IO_BUFFER_SIZE,
                                                fileDateSize), 2, "File Date/Size");
    viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_ORIGINAL_LOOP_TIME,
                                                originalLoopTime), 3, "Original Loop Time");

	viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_ORIGINAL_TS_DATA_RATE,
												RSDVSG_IO_BUFFER_SIZE,
                                                originalTSDataRate), 4, "Original TS Data Rate");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player File Start Stop Position
/* Purpose:  This function configures the loaded player file start and stop 
/*           positions.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerFileStartStopPosition(
	ViSession	instrSession,
	ViReal64	playerFileStartPosition,
	ViReal64	playerFileStopPosition
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_PLAYER_FILE_START_POSITION,
                                                playerFileStartPosition), 2, "Player File Start Position");
	   
    viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_PLAYER_FILE_STOP_POSITION,
                                                playerFileStopPosition), 3, "Player File Stop Position");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Current Play Time
/* Purpose:  Queries the current play time in the loaded play file.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerCurrentPlayTime(
	ViSession	instrSession,
	ViReal64*	currentPlayTime
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_CURRENT_PLAY_TIME,
                                                currentPlayTime), 2, "Current Play Time");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player TV Standard
/* Purpose:  This function configures the TV standard.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerTVStandard(
	ViSession	instrSession,
	ViInt32	tvStandard
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_TV_STANDARD,
                                               tvStandard), 2, "TV Standard");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Service
/* Purpose:  This function selects a service in the service list.
 *===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerService(
	ViSession	instrSession,
	ViInt32	service
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_SERVICE_LIST_INDEX,
                                               service), 2, "Service");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Service List Maximum Number
/* Purpose:  This function queries the maximum number of services in the 
/*           current service list.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerServiceListMaximumNumber(
	ViSession	instrSession,
	ViInt32*	maximumNumber
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
  
	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_SERVICE_LIST_MAXIMUM_NUMBER,
                                               maximumNumber), 2, "Maximum Number");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Service List Info
/* Purpose:  This function queries the content at position <numsuffix> in 
/*           the current service list.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerServiceListInfo(
	ViSession	instrSession,
	ViInt32	serviceList,
	ViInt32	bufferSize,
	ViChar	info[]
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	repcap[RSDVSG_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsdvsg_invalidViInt32Range (serviceList, 1, 99) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Service List");
    }
	if (info == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Info");
    
    sprintf (repcap, "L%ld", serviceList);  

	viCheckErr (rsdvsg_GetAttributeViString (instrSession, repcap, RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_SERVICE_LIST_INFO,
                                            bufferSize, info));
                                              
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Video
/* Purpose:  This function selects a video elementary stream in the video 
/*           elementary stream list.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerVideo(
	ViSession	instrSession,
	ViInt32	video
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_VIDEO_LIST_INDEX,
                                               video), 2, "Video");
  Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Video List Maximum Number
/* Purpose:  This function queries the maximum number of video elementary 
/*           streams in the current video elementary stream list. 
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVideoListMaximumNumber(
	ViSession	instrSession,
	ViInt32*	maximumNumber
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_LIST_MAXIMUM_NUMBER,
                                               maximumNumber), 2, "Maximum Number");
  Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Video List Info
/* Purpose:  This function queries the content at position <numsuffix> in 
/*           the current video elementary stream list.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVideoListInfo(
	ViSession	instrSession,
	ViInt32	videoElementaryStream,
	ViInt32	bufferSize,
	ViChar	info[]
)
{
    ViStatus	error = VI_SUCCESS;
   	ViChar	repcap[RSDVSG_IO_BUFFER_SIZE] = ""; 
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsdvsg_invalidViInt32Range (videoElementaryStream, 1, 99) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Service List");
    }
	if (info == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Info");
    
    sprintf (repcap, "L%ld", videoElementaryStream);  

	viCheckErr (rsdvsg_GetAttributeViString (instrSession, repcap, RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_LIST_INFO,
                                            bufferSize, info));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Video Parameters
/* Purpose:  This function returns the AV Signal Player video parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVideoParameters(
	ViSession	instrSession,
	ViChar	coding[],
	ViChar	esDataRate[],
	ViChar 	frameRate[],
	ViChar	pictureSize[],
	ViChar	pictureFormat[]
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_CODING,
												RSDVSG_IO_BUFFER_SIZE,
                                                coding), 2, "Coding");
    viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_ES_DATA_RATE,
                                                RSDVSG_IO_BUFFER_SIZE, esDataRate), 3, "ES Data Rate");
	viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_FRAME_RATE,
                                               RSDVSG_IO_BUFFER_SIZE, frameRate), 4, "Frame Rate");
	viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_PICTURE_SIZE,
												RSDVSG_IO_BUFFER_SIZE,
                                                pictureSize), 5, "Picture Size");
	viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_VIDEO_PICTURE_FORMAT,
												RSDVSG_IO_BUFFER_SIZE,
                                                pictureFormat), 6, "Picture Format");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Audio
/* Purpose:  This function selects a audio elementary stream in the audio 
/*           elementary stream list.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerAudio(
	ViSession	instrSession,
	ViInt32	audio
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_AUDIO_LIST_INDEX,
                                               audio), 2, "Audio");
    Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Audio List Maximum Number
/* Purpose:  This function queries the maximum number of audio elementary 
/*           streams in the current audio elementary stream list. 
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerAudioListMaximumNumber(
	ViSession	instrSession,
	ViInt32*	maximumNumber
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_LIST_MAXIMUM_NUMBER,
                                               maximumNumber), 2, "Maximum Number");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Audio List Info
/* Purpose:  This function queries the content at position <numsuffix> in 
/*           the current audio elementary stream list.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerAudioListInfo(
	ViSession	instrSession,
	ViInt32	audioElementaryStream,
	ViInt32	bufferSize,
	ViChar	info[]
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	repcap[RSDVSG_IO_BUFFER_SIZE] = ""; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsdvsg_invalidViInt32Range (audioElementaryStream, 1, 99) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Service List");
    }
	if (info == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Info");
    
    sprintf (repcap, "L%ld", audioElementaryStream);  

	viCheckErr (rsdvsg_GetAttributeViString (instrSession, repcap, RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_LIST_INFO,
                                            bufferSize, info));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Audio Parameters
/* Purpose:  This function returns the AV Signal Player audio parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerAudioParameters(
	ViSession	instrSession,
	ViChar	coding[],
	ViChar	esDataRate[],
	ViInt32*	samplingRate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));


    viCheckParm (rsdvsg_GetAttributeViString (instrSession, "", RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_CODING, 
                        RSDVSG_IO_BUFFER_SIZE, coding), 2, "Coding");
    
    viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_ES_DATA_RATE,
                                                RSDVSG_IO_BUFFER_SIZE, esDataRate), 3, "ES Data Rate");
   	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_AUDIO_SAMPLING_RATE,
                                               samplingRate), 4, "Sampling Rate");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Video Resolution
/* Purpose:  This function configures the AV Signal Player video resolution.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerVideoResolution(
	ViSession	instrSession,
	ViInt32	videoResolution
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_VIDEO_RESOLUTION,
                                               videoResolution), 2, "Video Resolution");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Video Aspect Ratio
/* Purpose:  This function configures the AV Signal Player video aspect 
/*           ratio.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerVideoAspectRatio(
	ViSession	instrSession,
	ViInt32	videoAspectRatio
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_VIDEO_ASPECT_RATIO,
                                               videoAspectRatio), 2, "videoAspectRatio");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface Conformity
/* Purpose:  This function returns the current standard conformity of the 
/*           selected signal.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceConformity(
	ViSession	instrSession,
	ViInt32	signalStandard,
	ViBoolean*	conformity
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsdvsg_invalidViInt32Range (signalStandard, RSDVSG_VAL_AV_SIGNAL_HDMI, RSDVSG_VAL_AV_SIGNAL_TS) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Signal Standard");
    }

    sprintf (buffer, "%s", signalStandardArr[signalStandard]);  

    viCheckParm (rsdvsg_GetAttributeViBoolean (instrSession, buffer,
                                                 RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_CONFORMITY,
                                                 conformity), 2, "Conformity");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Interface HDMI
/* Purpose:  This function configures the AV Signal Player interface 
/*           settings for HDMI and DVI.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceHDMI(
	ViSession	instrSession,
	ViBoolean	hdcp,
	ViBoolean	dvi
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_HDMI_HDCP,
                                                 hdcp), 2, "HDCP");

	viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_HDMI_DVI,
                                                 dvi), 3, "DVI");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface HD SDI
/* Purpose:  This function returns the AV Signal Player interface settings 
/*           for HD-SDI.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceHDSDI(
	ViSession	instrSession,
	ViInt32*	resolution,
	ViBoolean*	embeddedAudio
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                              RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_HD_SDI_RESOLUTION,
                                               resolution), 2, "Resolution");
    viCheckParm (rsdvsg_GetAttributeViBoolean (instrSession, "",
                                                RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_HD_SDI_EMBEDDED_AUDIO,
                                                 embeddedAudio), 3, "Embedded Audio");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Interface Composite
/* Purpose:  This function configures the AV Signal Player interface 
/*           settings for composite.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceComposite(
	ViSession	instrSession,
	ViInt32	ntscPedestal,
	ViBoolean	mVision
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_COMPOSITE_NTSC_PEDESTAL,
                                               ntscPedestal), 2, "NTSC Pedestal");
    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_COMPOSITE_MACRO_VISION,
                                                 mVision), 3, "Macro Vision");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface Composite
/* Purpose:  This function returns the AV Signal Player interface settings 
/*           for composite.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceComposite(
	ViSession	instrSession,
	ViInt32*	colorSystem
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_COLOR_SYSTEM,
                                               colorSystem), 2, "Color System");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Interface Component (RGB Sync jiny cmd)
/* Purpose:  This function configures the AV Signal Player interface 
/*           settings for component.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceComponent(
	ViSession	instrSession,
	ViInt32	color,
	ViInt32	rgbSync
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_COMPONENT,
                                               color), 2, "Color");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface Component
/* Purpose:  This function returns the AV Signal Player interface settings 
/*           for component.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceComponent(
	ViSession	instrSession,
	ViInt32*	hdtvSync
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_COMPONENT_HDTV_SYNC,
                                               hdtvSync), 2, "HDTV Sync");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Player Interface Scart
/* Purpose:  This function configures the AV Signal Player interface 
/*           settings for SCART.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalPlayerInterfaceScart(
	ViSession	instrSession,
	ViInt32	monitorInputSelect,
	ViInt32	formatWSS
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_MONITOR_INPUT_SELECT,
                                               monitorInputSelect), 2, "Monitor Input Select");
	
	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_PLAYER_INTERFACE_SCART_FORMAT_WSS,
                                               formatWSS), 3, "Format WSS");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface YC
/* Purpose:  This function returns the AV Signal Player interface settings 
/*           for Y/C.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceYC(
	ViSession	instrSession,
	ViInt32*	colorSystem
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_YC,
                                               colorSystem), 2, "Color System");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface VGA
/* Purpose:  This function returns the AV Signal Player interface settings 
/*           for VGA.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceVGA(
	ViSession	instrSession,
	ViInt32*	colorSystem
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_VGA,
                                               colorSystem), 2, "Color System");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface TS
/* Purpose:  This function returns the AV Signal Player interface settings 
/*           for TS.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceTS(
	ViSession	instrSession,
	ViInt32*	packetLength,
	ViInt32*	asi
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_TS_PACKET_LENGTH,
                                               packetLength), 2, "Packet Length");
	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_TS_ASI,
                                               asi), 3, "ASI");	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player Interface RGB Sync
/* Purpose:  This function returns the AV Signal Player interface RGB sync.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerInterfaceRGBSync(
	ViSession	instrSession,
	ViInt32*	rgbSync
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_PLAYER_INTERFACE_RGB_SYNC,
                                               rgbSync), 2, "RGB Sync");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player HDMI EDID
/* Purpose:  This function returns the AV Signal Player VESA Extended 
/*           Display Identification Data (EDID) of the connected HDMI device.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerHDMIEDID(
	ViSession	instrSession,
	ViInt32	bufferSize,
	ViChar	edidHDMI[]
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	sprintf(buffer, "AVPL:READ:EDID:HDMI? ASC");
	
	viCheckErr( rsdvsg_QueryViString (instrSession,
                                          buffer,
                                          bufferSize,
                                          edidHDMI));	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Player VGA EDID
/* Purpose:  This function returns the AV Signal Player VESA Extended 
/*           Display Identification Data (EDID) of the connected VGA device.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalPlayerVGAEDID(
	ViSession	instrSession,
	ViInt32	bufferSize,
	ViChar	edidVGA[]
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	sprintf(buffer, "AVPL:READ:EDID:VGA? ASC");
	
	viCheckErr( rsdvsg_QueryViString (instrSession,
                                          buffer,
                                          bufferSize,
                                          edidVGA));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: AV Signal Generator Mode
/* Purpose:  This function selects the AV Signal Generator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_AVSignalGeneratorMode(
	ViSession	instrSession
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsdvsg_SetAttributeViString (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_MODE, NULL));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Status
/* Purpose:  Select the status of the AV Generator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorStatus(
	ViSession	instrSession,
	ViInt32	command
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_STATUS,
                                               command), 2, "Command");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Status
/* Purpose:  Queries the actual state of the AV Generator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorStatus(
	ViSession	instrSession,
	ViInt32*	command
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_STATUS,
                                               command), 2, "Command");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Source
/* Purpose:  Selects the file to be loaded into the AV Signal Generator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorSource(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_SOURCE,
                                                fileName), 2, "File Name");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Select AV Signal Generator Video Frame
/* Purpose:  Selects the active video frame in PAUSE mode.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_SelectAVSignalGeneratorVideoFrame(
	ViSession	instrSession,
	ViInt32	videoFrame
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
  
	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_FRAME,
                                               videoFrame), 2, "Video Frame");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Video Frame Rate
/* Purpose:  This function sets the frame rate that is used to reproduce the 
/*           test patterns. The available setting data depends on the picture 
/*           size and is adapted automatically. If natural video sequences 
/*           were used as templates, they are marked separately. 
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorVideoFrameRate(
	ViSession	instrSession,
	ViReal64	frameRate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
       
	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_FRAME_RATE,
                                                frameRate), 2, "Frame Rate");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Audio Sampling Rate
/* Purpose:  This function configures the sample rate of the contained audio 
/*           signal. 
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorAudioSamplingRate(
	ViSession	instrSession,
	ViInt32	samplingRate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_SAMPLING_RATE,
                                               samplingRate), 2, "Sampling Rate");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Pixelshift
/* Purpose:  This function scrolls the screen contents with the selected 
/*           velocity and direction. 
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorPixelshift(
	ViSession	instrSession,
	ViBoolean	pixelshift,
	ViInt32	verticalPixelshift
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_GENERATOR_PIXELSHIFT,
                                                 pixelshift), 2, "Pixelshift");
	if (pixelshift == VI_TRUE)
		viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VERTICAL_PIXELSHIFT,
                                               verticalPixelshift), 3, "Vertical Pixelshift");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator RGB Video
/* Purpose:  This function configures the AV Signal Generator video settings 
/*           for RGB signals.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorRGBVideo(
	ViSession	instrSession,
	ViInt32	adjustment,
	ViInt32	resolutionClipping,
	ViReal64	overallAmplification,
	ViReal64	rLevel,
	ViReal64	gLevel,
	ViReal64	bLevel,
	ViReal64	offset
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_ADJUSTMENT,
                                               adjustment), 2, "Adjustment");	
	
	if (adjustment == RSDVSG_VAL_AVG_VIDEO_ADJUSTMENT_VARIABLE) {
		viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_RESOLUTION_CLIPPING,
                                               resolutionClipping), 3, "Resolution Clipping");
    	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_OVERALL_AMPLIFICATION,
                                                overallAmplification), 4, "Overall Amplification");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_R_LEVEL,
                                                rLevel), 5, "R Level");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_G_LEVEL,
                                                gLevel), 6, "G Level");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_RGB_VIDEO_B_LEVEL,
                                                bLevel), 7, "B Level");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_OFFSET,
                                                offset), 8, "Offset");		
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator YCrCb Video
/* Purpose:  This function configures the AV Signal Generator video settings 
/*           for YCrCb signals.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorYCrCbVideo(
	ViSession	instrSession,
	ViInt32	adjustment,
	ViInt32	resolutionClipping,
	ViReal64	overallAmplification,
	ViReal64	chromaAmplification,
	ViReal64	yLevel,
	ViReal64	cbpbLevel,
	ViReal64	crprLevel,
	ViReal64	offset
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_ADJUSTMENT,
                                               adjustment), 2, "Adjustment");	
	
	if (adjustment == RSDVSG_VAL_AVG_VIDEO_ADJUSTMENT_VARIABLE) {
		viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_RESOLUTION_CLIPPING,
                                               resolutionClipping), 3, "Resolution Clipping");
    	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_OVERALL_AMPLIFICATION,
                                                overallAmplification), 4, "Overall Amplification");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_CHROMA_AMPLIFICATION,
                                                chromaAmplification), 5, "Chroma Amplification");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_Y_LEVEL,
                                                yLevel), 6, "Y Level");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_CB_PB_LEVEL,
                                                cbpbLevel), 7, "Cb/Pb Level");
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_YCRCB_VIDEO_CR_PR_LEVEL,
                                                crprLevel), 8, "Cr/Pr Level");		
		viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_VIDEO_OFFSET,
                                                offset), 9, "Offset");		
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Audio
/* Purpose:  This function configures the audio settings for the AV Signal 
/*           Generator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorAudio(
	ViSession	instrSession,
	ViInt32	audioSource,
	ViInt32	channelAllocation,
	ViInt32	resolution,
	ViReal64	audioLevel
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_SOURCE,
                                               audioSource), 2, "Audio Source");
    if (audioSource >= RSDVSG_VAL_AVG_AUDIO_SOURCE_SINE_SINGLE_FREQ){
		
		
		viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
	                                            RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_CHANNEL_ALLOCATION,
	                                            channelAllocation), 3, "Channel Allocation");
		viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
	                                            RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_RESOLUTION,
                                                resolution), 4, "Resolution");
	}
	
	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_LEVEL,
                                                audioLevel), 5, "Audio Level");
	
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Audio Frequency
/* Purpose:  This function configures the AV Signal Generator audio 
/*           frequency of channel 1 to 8 of the sine wave generator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorAudioFrequency(
	ViSession	instrSession,
	ViInt32	channel,
	ViReal64 channelFrequency
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsdvsg_invalidViInt32Range (channel, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    }

    sprintf (buffer, "CH%ld", channel);  

    viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, buffer,
                                                RSDVSG_ATTR_AV_SIGNAL_GENERATOR_AUDIO_FREQUENCY,
                                                channelFrequency), 3, "Channel Frequency");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface Conformity
/* Purpose:  This function returns the current standard conformity of the 
/*           selected signal.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceConformity(
	ViSession	instrSession,
	ViInt32	signalStandard,
	ViBoolean*	conformity
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    if (rsdvsg_invalidViInt32Range (signalStandard, RSDVSG_VAL_AV_SIGNAL_HDMI, RSDVSG_VAL_AV_SIGNAL_TS) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Signal Standard");
    }

    sprintf (buffer, "%s", signalStandardArr[signalStandard]);  

    viCheckParm (rsdvsg_GetAttributeViBoolean (instrSession, buffer,
                                                 RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_CONFORMITY,
                                                 conformity), 3, "Conformity");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Interface HDMI
/* Purpose:  This function configures the AV Signal Generator interface 
/*           settings for HDMI and DVI.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceHDMI(
	ViSession	instrSession,
	ViBoolean	hdcp,
	ViBoolean	dvi
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_HDMI_HDCP,
                                                 hdcp), 2, "HDCP");

	viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_HDMI_DVI,
                                                 dvi), 3, "DVI");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Interface Composite
/* Purpose:  This function configures the AV Signal Generator interface 
/*           settings for composite.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceComposite(
	ViSession	instrSession,
	ViInt32	ntscPedestal,
	ViBoolean	mVision
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_COMPOSITE_NTSC_PEDESTAL,
                                                 ntscPedestal), 2, "NTSC Pedestal");

	viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_COMPOSITE_MACRO_VISION,
                                                 mVision), 3, "Macro Vision");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface Composite
/* Purpose:  This function returns the AV Signal Generator interface 
/*           settings for composite.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceComposite(
	ViSession	instrSession,
	ViInt32*	colorSystem
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_COMPOSITE,
                                               colorSystem), 2, "Color System");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Interface Component (RGB Sync jiny cmd)
/* Purpose:  This function configures the AV Signal Generator interface 
/*           settings for component.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceComponent(
	ViSession	instrSession,
	ViInt32	color,
	ViInt32	rgbSync
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_COMPONENT,
                                               color), 2, "Color");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface Component
/* Purpose:  This function returns the AV Signal Generator interface 
/*           settings for component.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceComponent(
	ViSession	instrSession,
	ViInt32*	hdtvSync
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_COMPONENT_HDTV_SYNC,
                                               hdtvSync), 2, "HDTV Sync");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface YC
/* Purpose:  This function returns the AV Signal Generator interface 
/*           settings for Y/C.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceYC(
	ViSession	instrSession,
	ViInt32*	colorSystem
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_YC,
                                               colorSystem), 2, "Color System");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure AV Signal Generator Interface Scart
/* Purpose:  This function configures the AV Signal Generator interface 
/*           settings for SCART.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureAVSignalGeneratorInterfaceScart(
	ViSession	instrSession,
	ViInt32	monitorInputSelect,
	ViInt32	formatWSS
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_SCART_MONITOR_INPUT_SELECT,
                                               monitorInputSelect), 2, "Monitor Input Select");
	
    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_AV_SIGNAL_GENERATOR_INTERFACE_SCART_FORMAT_WSS,
                                               formatWSS), 3, "Format WSS");
  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface VGA
/* Purpose:  This function returns the AV Signal Generator interface 
/*           settings for VGA.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceVGA(
	ViSession	instrSession,
	ViInt32*	colorSystem
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_VGA,
                                               colorSystem), 2, "Color System");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface TS
/* Purpose:  This function returns the AV Signal Generator interface 
/*           settings for TS.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceTS(
	ViSession	instrSession,
	ViInt32*	packetLength,
	ViInt32*	asi
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_TS_PACKET_LENGTH,
											  packetLength ), 2, "Packet Length");
                                               
	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_ASI,
                                               asi), 3, "ASI");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator Interface RGB Sync
/* Purpose:  This function queries the AV Signal Generator interface RGB 
/*           sync.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorInterfaceRGBSync(
	ViSession	instrSession,
	ViInt32*	rgbSync
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_AV_SIGNAL_GENERATOR_INTERFACE_RGB_SYNC,
											    rgbSync), 2, "RGB Sync");  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator HDMI EDID
/* Purpose:  This function returns the AV Signal Generator VESA Extended 
/*           Display Identification Data (EDID) of the connected HDMI device.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorHDMIEDID(
	ViSession	instrSession,
	ViInt32	bufferSize,
	ViChar edidHDMI[]
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	sprintf(buffer, "AVG:READ:EDID:HDMI? ASC");
	
	viCheckErr( rsdvsg_QueryViString (instrSession,
                                          buffer,
                                          bufferSize,
                                          edidHDMI));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query AV Signal Generator VGA EDID
/* Purpose:  This function returns the AV Signal Generator VESA Extended 
/*           Display Identification Data (EDID) of the connected VGA device.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryAVSignalGeneratorVGAEDID(
	ViSession	instrSession,
	ViInt32	bufferSize,
	ViChar	edidVGA[]
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	buffer[RSDVSG_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	
	sprintf(buffer, "AVG:READ:EDID:VGA? ASC");
	
	viCheckErr( rsdvsg_QueryViString (instrSession,
                                          buffer,
                                          bufferSize,
                                          edidVGA));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: TS Player Mode
/* Purpose:  This function selects the TS Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_TSPlayerMode(
	ViSession	instrSession
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

  viCheckErr (rsdvsg_SetAttributeViString (instrSession, "",
               						RSDVSG_ATTR_TS_PLAYER_MODE, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Application
/* Purpose:  TS Player and Recorder are sharing the same resources and 
/*           therefore only one of these applications are available at the 
/*           same time. The TS-Player is selectable only if the TS-Recorder 
/*           is stopped. The same applies to the TS-Player. The TS-Recorder 
/*           is only available when the TS-Player is stopped.This function 
/*           makes the preselection on one of the applications. Possible 
/*           selections are TS Player (PLAY) or TS Record (RECord).
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSApplication(
	ViSession	instrSession,
	ViInt32	application
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_APPLICATION,
                                               application), 2, "Application");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player and Recorder Status
/* Purpose:  Select the status of the TSPlayer/Recorder.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerRecorderStatus(
	ViSession	instrSession,
	ViInt32	command
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_AND_RECORDER_STATUS,
                                               command), 2, "Command");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query TS Player and Recorder Status
/* Purpose:  Queries the actual state of the TSPlayer or Recorder.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryTSPlayerRecorderStatus(
	ViSession	instrSession,
	ViInt32*	command
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_QUERY_TS_PLAYER_AND_RECORDER_STATUS,
                                               command), 2, "Command");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Source
/* Purpose:  Selects the file to be loaded into the TS Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerSource(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

 	viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_SOURCE,
                                                fileName), 2, "File Name");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query TS Player Source Parameters
/* Purpose:  This function reads player parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryTSPlayerSourceParameters(
	ViSession	instrSession,
	ViChar	fileDateSize[],
	ViReal64*	originalLoopTime,
	ViInt32*	originalTSDataRate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
      
	viCheckParm (rsdvsg_GetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_PLAYER_FILE_DATE_SIZE,
												RSDVSG_IO_BUFFER_SIZE,
                                                fileDateSize), 2, "File Date/Size");
    viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_PLAYER_ORIGINAL_LOOP_TIME,
                                                originalLoopTime), 3, "Original Loop Time");

	viCheckParm (rsdvsg_GetAttributeViInt32 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_PLAYER_ORIGINAL_TS_DATA_RATE,
                                                originalTSDataRate), 4, "Original TS Data Rate");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query TS Player Current Play Time
/* Purpose:  Returns the present position of the progress bar.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryTSPlayerCurrentPlayTime(
	ViSession	instrSession,
	ViReal64*	currentPlayTime
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_PLAYER_CURRENT_PLAY_TIME,
                                                currentPlayTime), 2, "Current Play Time");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Timing Settings
/* Purpose:  This function configures timing settings.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerTimingSettings(
	ViSession	instrSession,
	ViReal64	tsDataRate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_TIMING_SETTINGS_TS_DATA_RATE,
                                                tsDataRate), 2, "TS Data Rate");
	   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Play Window Settings
/* Purpose:  This function configures timing settings. Use with .trp files, 
/*           when playback is stopped.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerPlayWindowSettings (ViSession instrSession,
                                                              ViReal64 playWindowStart,
                                                              ViReal64 playWindowStop)
{
	ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_TIMING_SETTINGS_PLAY_WINDOW_START,
                                                playWindowStart), 2, "Play Window Start");
	 
	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_TIMING_SETTINGS_PLAY_WINDOW_STOP,
                                                playWindowStop), 3, "Play Window Stop");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Byte Rate
/* Purpose:  This function sets the TS byte rate.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerByteRate(
	ViSession	instrSession,
	ViReal64	byteRate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_BYTE_RATE,
												 byteRate), 2, "Byte Rate");  
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Nullpacket Stuffing
/* Purpose:  This function sets the output data rate of the TS Player.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerNullpacketStuffing(
	ViSession	instrSession,
	ViBoolean	nullpacketStuffing
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_TS_PLAYER_NULLPACKET_STUFFING,
                                                 nullpacketStuffing), 2, "Nullpacket Stuffing");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player PCR Jitter
/* Purpose:  This function configures PCR Jitter settings.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerPCRJitter(
	ViSession	instrSession,
	ViBoolean	pcrJitter,
	ViInt32	waveform,
	ViReal64	amplitude,
	ViReal64	frequency
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_PCR_JITTER,
                                                pcrJitter), 2, "PCR Jitter");        
	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_PCR_JITTER_WAVEFORM,
                                                 waveform), 3, "Waveform");
	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_PCR_JITTER_AMPLITUDE,
                                                amplitude), 4, "Amplitude");
	viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_PLAYER_PCR_JITTER_FREQUENCY,
                                                frequency), 5, "Frequency");	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Seamless Loop
/* Purpose:  This function configures seamless loop parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerSeamlessLoop(
	ViSession	instrSession,
	ViBoolean	continuityCounter,
	ViBoolean	pcrDTSPTS,
	ViBoolean	tdttot
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_TS_PLAYER_SEAMLESS_LOOP_CONTINUITY_COUNTER,
                                                 continuityCounter), 2, "Continuity Counter");

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_TS_PLAYER_SEAMLESS_LOOP_PCR_DTS_PTS,
                                                 pcrDTSPTS), 3, "PCR DTS/PTS");

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_TS_PLAYER_SEAMLESS_LOOP_TDT_TOT,
                                                 tdttot), 4, "TDT/TOT");	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Interface Settings (GEN)
/* Purpose:  This function configures generator interface.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerInterfaceSettings(
	ViSession	instrSession,
	ViInt32	packetLength,
	ViInt32	tsSerialOutput
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
      
	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_INTERFACE_SETTINGS_PACKET_LENGTH,
                                               packetLength), 2, "Packet Length");
	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_INTERFACE_SETTINGS_TS_SERIAL_OUTPUT,
                                               tsSerialOutput), 3, "TS Serial Output");	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Interface Settings Stop Data (*D na pristroji je, ale nefu)
/* Purpose:  This function determines whether the TS player generates data 
/*           when it is in Pause or Stop mode.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerInterfaceSettingsStopData(
	ViSession	instrSession,
	ViInt32	stopData
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

      viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_INTERFACE_SETTINGS_STOP_DATA,
                                               stopData), 2, "Stop Data");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Player Nullpacket Settings (*D - na pristroji je, ale nefunguje)
/* Purpose:  This function configures TSGEN packet settings.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSPlayerNullpacketSettings(
	ViSession	instrSession,
	ViInt32	nullpackets,
	ViInt32	payload,
	ViInt32	prbs
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_NULLPACKET,
											    nullpackets), 2, "Nullpackets");     
    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_PAYLOAD,
											    payload), 3, "Payload");    
	viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_PLAYER_PRBS,
											    prbs), 4, "PRBS");     
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: New TS Recorder Stream
/* Purpose:  Selects the file to be created for the record content. File 
/*           extension is set automatically and depends on the selected 
/*           record input.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_NewTSRecorderStream(
	ViSession	instrSession,
	ViString	fileName
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_NEW_TS_RECORDER_STREAM,
                                                fileName), 2, "FileName");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query TS Recorder Parameters
/* Purpose:  This function reads recorder parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryTSRecorderParameters(
	ViSession	instrSession,
	ViReal64*	freeMemory,
	ViReal64*	inputTSDataRate,
	ViReal64*	recordLength
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_RECORDER_FREE_MEMORY,
                                                freeMemory), 2, "Free Memory");
    viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_RECORDER_INPUT_TS_DATA_RATE,
                                                inputTSDataRate), 3, "Input TS Data Rate");

    viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_RECORDER_RECORD_LENGTH,
                                                recordLength), 4, "Record Length");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Query TS Recorder Progress
/* Purpose:  This function queries the current record progress in %.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_QueryTSRecorderProgress(
	ViSession	instrSession,
	ViReal64*	recordProgress
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));


    viCheckParm (rsdvsg_GetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_QUERY_TS_RECORDER_PROGRESS,
                                                recordProgress), 2, "Record Progress");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Configure TS Recorder Interface Settings
/* Purpose:  This function configures recorder interface.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureTSRecorderInterfaceSettings(
	ViSession	instrSession,
	ViInt32	input,
	ViReal64	recordSize,
	ViInt32	recordMode
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_RECORDER_INTERFACE_SETTINGS_INPUT,
                                               input), 2, "Input");
    viCheckParm (rsdvsg_SetAttributeViReal64 (instrSession, "",
                                                RSDVSG_ATTR_TS_RECORDER_INTERFACE_SETTINGS_RECORD_SIZE,
                                                recordSize), 3, "Record Size");
    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_TS_RECORDER_INTERFACE_SETTINGS_RECORD_MODE,
                                               recordMode), 4, "Record Mode");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: File Manager Operations
/* Purpose:  The MMEMory (mass memory) subsystem function provides commands 
/*           which allow for access to the storage media of the instrument 
/*           and for storing and loading various instrument settings.
/*           The File to be Printed/Plotted command stores the HCOPy outputs 
/*           in a file. The various drives can be addressed via the mass 
/*           storage unit specifier <msus> using the conventional DOS syntax. 
/*           The internal hard disk is addressed by "C:", the floppy-disk 
/*           drive installed by "A:". The file names <file_name> are 
/*           indicated as string parameters with the commands being enclosed 
/*           in quotation marks. They correspond to the DOS conventions.
/*           DOS file names consist of max. 8 ASCII characters and an 
/*           extension of up to three characters separated from the file name 
/*           by a colon "." Both, the colon and the extension are optional. 
/*           The colon is not part of the file name. DOS file names do not 
/*           differ between uppercase and lowercase notation. All letters and 
/*           digits are permitted as well as the special characters "_", "^", 
/*           "$", "~", "!", "#", "%", "&", "-", "{","}", "(", ")", "@" and "' 
/*           ". Reserved file names are CLOCK$, CON, AUX, COM1 to COM4, LPT1 
/*           to LPT3, NUL and PRN. The two characters "*" and "?" have the 
/*           function of so-called "wildcards", i.e., they are variables for
/*           selection of several files. The question mark "?" replaces 
/*           exactly one character which may be any, the asterisk means any 
/*           of the remaining characters in the file name. "*.*" thus means 
/*           all files in a directory.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_FileManagerOperations(
	ViSession	instrSession,
	ViInt32	operation,
	ViString	source,
	ViString	destination
)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));


 	if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    
	switch (operation){
        case RSDVSG_VAL_FILE_NEW:
			viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_FILE_MANAGER_MAKE_DIRECTORY,
                                                source), 3, "Source");
        break;
		case RSDVSG_VAL_FILE_COPY:
            viCheckErr( viPrintf (instrSession, "MMEM:COPY '%s','%s'", source, destination)); 
        break;
        case RSDVSG_VAL_FILE_RENAME:
            viCheckErr( viPrintf (instrSession, "MMEM:MOVE '%s','%s'", source, destination)); 
        break;
        case RSDVSG_VAL_FILE_DELETE:
			viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_FILE_MANAGER_DELETE_FILE,
                                                source), 3, "Source");
        break;
        case RSDVSG_VAL_FILE_RDIR:
			viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_FILE_MANAGER_DELETE_DIRECTORY,
                                                source), 3, "Source");
        break;
        case RSDVSG_VAL_FILE_CDIR:
			viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_FILE_MANAGER_CHANGE_DIRECTORY,
                                                source), 3, "Source");
        break;        
        case RSDVSG_VAL_FILE_CDISC:
			viCheckParm (rsdvsg_SetAttributeViString (instrSession, "",
                                                RSDVSG_ATTR_FILE_MANAGER_CHANGE_DRIVES,
                                                source), 3, "Source");
        break; 
		
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Operation");
        }
	
    viCheckErr(rsdvsg_CheckStatus (instrSession));
    }

 Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: File Directory Path
/* Purpose:  This function reads out the subdirectories or subdirectories 
/*           and files in the specified directory. If no directory is 
/*           specified, the default directory selected with the MMEM:CDIR 
/*           command is read out on the default drive selected with the 
/*           MMEM:MSIS command. 
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_FileDirectoryPath (ViSession instrSession, ViInt32 read,
                                        ViString directory, ViInt32 bufferSize,
                                        ViChar output[], ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0)
	{

	    if (bufferSize != 0 && output == VI_NULL)
	        return VI_ERROR_PARAMETER5;

	    if (read == RSDVSG_VAL_MMEM_CATALOG_FILES)
	    {
	        viCheckErr( viPrintf(instrSession, "*CLS;:MMEM:CAT? '%s'\n", directory));
	    }
	    else
	        viCheckErr( viPrintf(instrSession, "*CLS;:MMEM:DCAT? '%s'\n", directory)); 
    
    
	    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &count)); 
	    checkErr( rsdvsg_CheckStatus (instrSession));
	    //strncpy(output, pbuffer, (size_t) bufferSize);
	    if (count>=(ViUInt32)bufferSize)
	    {
	        strncpy(output, pbuffer, (size_t) (bufferSize-1));
	        output[bufferSize-1]='\0';
	        error = (ViStatus) (count+1);
	    } 
	    else
	    {
	        strncpy(output, pbuffer, (size_t) count); 
	        output[count]='\0';   
	    }
		
		if (returnedValues) *returnedValues = (ViInt32) count;
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Catalog Length
/* Purpose:  This function reads out the number of files or subdirectories 
/*           in the specified directory. If no directory is specified, the 
/*           default directory selected with the MMEM:CDIR command is read 
/*           out on the default drive selected with the MMEM:MSIS command.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_CatalogLength (ViSession instrSession, ViInt32 read,
                                         ViInt32 *catalogLength)
{
   ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Catalog Length");
    
    if (read == RSDVSG_VAL_MMEM_CATALOG_FILES)
    {
        viCheckErr( viQueryf (instrSession, "MMEM:CAT:LENG?", "%ld", catalogLength));
    }
    else
        viCheckErr( viQueryf (instrSession, "MMEM:DCAT:LENG?", "%ld", catalogLength));
 }   
    viCheckErr(rsdvsg_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Reference Oscillator
/* Purpose:  This function configures reference oscillator.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_ConfigureReferenceOscillator(
	ViSession	instrSession,
	ViInt32	source
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "",
                                               RSDVSG_ATTR_REFERENCE_OSCILLATOR_SOURCE,
                                               source), 2, "Source");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: System Display Update
/* Purpose:  This function activates/deactivates updating of the display. 
/*           Upon changing to manual control, the status is automatically set 
/*           to ON.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_SystemDisplayUpdate(
	ViSession	instrSession,
	ViBoolean	displayUpdate
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_SYSTEM_DISPLAY_UPDATE,
                                                 displayUpdate), 2, "Display Update");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: System Keypad Lock
/* Purpose:  This function Locks or unlocks the front panel keypad of the 
/*           R&S DVSG including the LOCAL key.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_SystemKeypadLock(
	ViSession	instrSession,
	ViBoolean	keypadLock
)
{
    ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsdvsg_SetAttributeViBoolean (instrSession, "",
                                                 RSDVSG_ATTR_SYSTEM_KEYPAD_LOCK,
                                                 keypadLock), 2, "Keypad Lock");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/*===========================================================================*/
/* Function: Set Status Register
/* Purpose:  This function sets the Enable, NTransition and PTransition bits 
/*               of status questionable and operating registers.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_SetStatusRegister (ViSession instrSession,
                                           ViInt32 registerOperation,
                                           ViInt32 enable, ViInt32 PTransition,
                                           ViInt32 NTransition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSDVSG_VAL_PRESET_STAT_REG) || (registerOperation > RSDVSG_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
        
    if ((enable < 0) || (enable > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Enable");
    
    if ((PTransition < 0) || (PTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "PTransition");     
    
    if ((NTransition < 0) || (NTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "NTransition");     
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSDVSG_VAL_PRESET_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:PRES")); 
            break;
            
            case RSDVSG_VAL_QUEST_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:QUES:ENAB %ld;PTR %ld;NTR %ld",
                    enable, PTransition, NTransition)); 
            break;
            
            case RSDVSG_VAL_OPER_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:OPER:ENAB %ld;PTR %ld;NTR %ld",
                    enable, PTransition, NTransition)); 
            break;
        }
    }   
    
    viCheckErr(rsdvsg_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*===========================================================================*/
/* Function: Get Status Register
/* Purpose:  This function contains the commands for the querying status 
/*           reporting system.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_GetStatusRegister (ViSession instrSession,
                                           ViInt32 registerOperation,
                                           ViInt32 statusRegistersQuery,
                                           ViInt32* registerValue)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSDVSG_VAL_QUEST_STAT_REG) || (registerOperation > RSDVSG_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if ((statusRegistersQuery < RSDVSG_VAL_REG_CONDITION) || (statusRegistersQuery > RSDVSG_VAL_REG_EVENT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Register Operation");  
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSDVSG_VAL_QUEST_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:QUES:%s?", "%ld",
                    statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
            
            case RSDVSG_VAL_OPER_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:OPER:%s?", "%ld",
                    statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
        }
    }   

    viCheckErr(rsdvsg_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                            
/*===========================================================================*/
/* Function: Set User-definable Status Register
/* Purpose:  This function sets the Enable, NTransition and PTransition 
/*           user-definable bits of status questionable and operating 
/*           registers.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_SetUserDefStatusRegister (ViSession instrSession,
                                                  ViInt32 registerOperation,
                                                  ViInt32 userdefinableBit,
                                                  ViInt32 enable,
                                                  ViInt32 PTransition,
                                                  ViInt32 NTransition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSDVSG_VAL_QUEST_STAT_REG) || (registerOperation > RSDVSG_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");

    if (registerOperation == RSDVSG_VAL_QUEST_STAT_REG) {
        if ((userdefinableBit < 9) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    else {
        if ((userdefinableBit < 8) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    
    if ((enable < 0) || (enable > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Enable");
    
    if ((PTransition < 0) || (PTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "PTransition");     
    
    if ((NTransition < 0) || (NTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "NTransition");     
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {

            case RSDVSG_VAL_QUEST_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:QUES:BIT%ld:ENAB %ld;PTR %ld;NTR %ld",
                    userdefinableBit, enable, PTransition, NTransition)); 
            break;
            
            case RSDVSG_VAL_OPER_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:OPER:BIT%ld:ENAB %ld;PTR %ld;NTR %ld",
                    userdefinableBit, enable, PTransition, NTransition)); 
            break;
        }
    }   
    
    viCheckErr(rsdvsg_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*===========================================================================*/
/* Function: Get User-definable Status Register
/* Purpose:  This function contains the commands for the querying 
/*           user-definable bits status reporting system.
/*===========================================================================*/
ViStatus _VI_FUNC rsdvsg_GetUserDefStatusRegister (ViSession instrSession,
                                                  ViInt32 registerOperation,
                                                  ViInt32 userdefinableBit,
                                                  ViInt32 statusRegistersQuery,
                                                  ViInt32* registerValue)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSDVSG_VAL_QUEST_STAT_REG   ) || (registerOperation > RSDVSG_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if (registerOperation == RSDVSG_VAL_QUEST_STAT_REG) {
        if ((userdefinableBit < 9) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    else {
        if ((userdefinableBit < 8) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    
    if ((statusRegistersQuery < RSDVSG_VAL_REG_CONDITION   ) || (statusRegistersQuery > RSDVSG_VAL_REG_EVENT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Register Operation");  
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSDVSG_VAL_QUEST_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:QUES:BIT%ld:%s?", "%ld",
                    userdefinableBit, statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
            
            case RSDVSG_VAL_OPER_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:OPER:BIT%ld:%s?", "%ld",
                    userdefinableBit, statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
        }
    }   

    viCheckErr(rsdvsg_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          

/*****************************************************************************
 * Function: rsdvsg_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsdvsg_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rsdvsg_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rsdvsg_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rsdvsg_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rsdvsg_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rsdvsg_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsdvsg_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rsdvsg_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rsdvsg_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rsdvsg_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rsdvsg_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rsdvsg_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsdvsg_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rsdvsg_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rsdvsg_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rsdvsg_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rsdvsg_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*============================================================================/
 * Function: Store and Load Application
 * Purpose:  This function stores or recalls application settings in the
 *           specified file.
/============================================================================*/
ViStatus _VI_FUNC rsdvsg_StoreLoadApplication (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSDVSG_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((operation < RSDVSG_VAL_SAVE) || (operation > RSDVSG_VAL_RECALL))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Operation");
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */

        switch (operation){
            case RSDVSG_VAL_SAVE:
                 sprintf (buffer,"*SAV 1;:MMEMory:STORe:STATe 1,\"%s\"", source);
            break;
        
            case RSDVSG_VAL_RECALL:
                sprintf (buffer,":MMEMory:LOAD:STATe 1,\"%s\"", source);
            break;
        }   
    }
    viCheckErr( viPrintf (instrSession, "%s\n", buffer));
    viCheckErr(rsdvsg_CheckStatus (instrSession));       
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
     

/*============================================================================/
 * Function: Store and Load Instrument Data
 * Purpose:  This function stores or recalls instrument data.
/============================================================================*/
ViStatus _VI_FUNC rsdvsg_StoreLoadInstrumentData (ViSession instrSession,
                                                 ViInt32 operation,
                                                 ViInt32 stateNumber)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    switch (operation){
        case RSDVSG_VAL_SAVE:
             viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "", RSDVSG_ATTR_STORE_INSTRUMENT_DATA, stateNumber),
                         3, "State Number"); 
        break;
        
        case RSDVSG_VAL_RECALL:
             viCheckParm (rsdvsg_SetAttributeViInt32 (instrSession, "", RSDVSG_ATTR_RECALL_INSTRUMENT_DATA, stateNumber),
                         3, "State Number"); 
        break;
			
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Operation");
        }
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rsdvsg_DefaultInstrSetup (instrSession));
    checkErr( rsdvsg_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?;*OPC\n"));
        viCheckErr( rsdvsg_WaitForOPC (instrSession, 600000));  
        viCheckErr( viScanf (instrSession, "%hd", testResult));
        if (*testResult == 0)
            strcpy (testMessage, "Self-Test Passed.");
        else
            strcpy (testMessage, "Self-Test Failed.");
        }
    else
        {
        /* Simulate Self Test */
        *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rsdvsg_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSDVSG_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSDVSG_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsdvsg_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Option Checking                                                 
 * Purpose:  This function switches ON/OFF option checking of the instrument 
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_setCheckOption (ViSession instrSession,
                                         ViBoolean state)
{
    ViStatus	error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
	
	sessionProperties -> optionChecking = state;

	return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which 
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_SystemVersion (ViSession instrSession,
                                          ViInt32 length, 
                                          ViChar systemVersion[])

{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    
    viCheckParm( rsdvsg_GetAttributeViString (instrSession, "", 
                                                RSDVSG_ATTR_SYSTEM_VERSION, length,
                                                systemVersion),
                                                2, "System Version");
    checkErr( rsdvsg_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL); 
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSDVSG_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsdvsg_GetAttributeViString( instrSession, "", RSDVSG_ATTR_SYSTEM_ERROR, RSDVSG_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function reads all system messages and returns a list of 
 *           comma separated strings.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_ErrorList (ViSession instrSession,
                                      ViInt32 bufferSize, 
                                      ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (bufferSize < 0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size");
    if (errors == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Errors");

    viCheckErr (rsdvsg_GetAttributeViString( instrSession, "", RSDVSG_ATTR_SYSTEM_ALL_ERRORS, 
        bufferSize, errors));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
        /*=CHANGE:================================================================*
            Insert instrument driver specific error codes here.  Example:

            {rsdvsg_ERROR_TOO_MANY_SAMPLES,   "Sample Count cannot exceed 512."},
         *=============================================================END=CHANGE=*/
            RSDVSG_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE + 256]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
        {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }

    if (description == VI_NULL)
        {
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsdvsg_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsdvsg_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rsdvsg_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/*============================================================================/
 * Function: Read To File From Instrument
 * Purpose:  This function is used to read data from the instrument and
 *           write it to a user specified file.
/============================================================================*/
ViStatus _VI_FUNC rsdvsg_ReadToFileFromInstrument (ViSession instrSession, ViString source,
                ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt32    length = 0;
    ViChar      buffer[RSDVSG_IO_BUFFER_SIZE] = "";
    ViUInt16 interface = VI_INTF_GPIB;     
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!source)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source");
    if (!destination)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");    
    checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
    switch (interface)
    {
        case VI_INTF_ASRL: /* Serial Interface. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    viCheckErr (viPrintf(instrSession, ":MMEMory:DATA? '%s'\n", source));
    //Read header       
    viCheckErr (viRead(instrSession, (ViPBuf)buffer, 2,  &ret_count));
    if (sscanf (buffer, "#%1ld", &count) != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE); 
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, (ViUInt32) count, &ret_count));
    if (sscanf (buffer, "%lu", &length)  != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);     
    viCheckErr(Rs_ReadToFile(instrSession, destination, (ViInt32) length, RS_VAL_TRUNCATE, &count)); 
    //Read \n and \r
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, 2, &ret_count)); 
    if (count!=(ViInt32) length)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);         
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rsdvsg_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (interface)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;  
}

/*============================================================================/
 * Function: Write From File To Instrument
 * Purpose:  This function is used to read data from a user specified file
 *           and write it to the instrument.
/============================================================================*/
ViStatus _VI_FUNC rsdvsg_WriteFromFileToInstrument (ViSession instrSession, ViString source,
                ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt32    fileSize            = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!source)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Source");
    if (!destination)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Destination");  
        
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
    fileSize = (ViUInt32) ftell (sourceFile);
    (void) fseek (sourceFile, 0, SEEK_SET);

    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#%ld%ld", destination, digits, fileSize));
    
    viCheckErr (Rs_WriteFromFile(instrSession, source, (ViInt32) fileSize, 0, &count));
    if (count!=(ViInt32) fileSize)
        viCheckErr (RS_ERROR_WRITING_FILE);     
        
    checkErr( rsdvsg_CheckStatus (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    
    return error;
}



/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rsdvsg_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rsdvsg_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer [10] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    
    viCheckErr (viQueryf (instrSession, "%s\n", "%[^\r\n]", command, buffer));
    if (strstr (buffer, "OFF") != NULL)
        *value = VI_FALSE;
    else
        *value = VI_TRUE;
            
    checkErr( rsdvsg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsdvsg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsdvsg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsdvsg_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rsdvsg_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr (viWrite (instrSession, (ViBuf) "&GTL\n",5,NULL));
    checkErr( rsdvsg_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/*****************************************************************************
 * Function: rsdvsg_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rsdvsg_close or by an RS class driver.
 *****************************************************************************/
ViStatus _VI_FUNC rsdvsg_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    /* Do not lock here. The caller manages the lock. */

    if(instrSession != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        viPrintf (instrSession, "&GTL");
        
        (void) viClose (instrSession);
        (void) viClose (rmSession);
        }
Error:    
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           RSDVSG_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rsdvsg_CheckStatusCallback (ViSession instrSession)
{
    ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[5];

    //viCheckErr (viQueryf (instrSession, "*ESR?\n", "%hd", &esr));
    viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, 4, &ret_cnt));
    buffer[ret_cnt] = '\0';
    esr = (ViInt16) atol (buffer);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:

    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rsdvsg_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    /*

       Change this function to wait for operation complete for your specific
       instrument. This example function is based on the IEEE 488.2 Common
       System Commands. This function assumes that the instrument does not
       generate service requests for other reasons. This example performs
       the following operations:

       - Enables the service request event.
       - Sends a command that forces the instrument to generate a service
         request.
       - Waits for the service request.
       - Cleans up after the service request.

    */
    
#if 0 // --- Select most suitable method ---

    ViUInt16    statusByte;
    ViInt32     opcTimeout = 0;
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viEnableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL));
   
    // wait for SRQ
    viCheckErr( viWaitOnEvent (io, VI_EVENT_SERVICE_REQ, opcTimeout, VI_NULL, VI_NULL));
    viCheckErr( viDisableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE));

    // clean up after SRQ
    viCheckErr( viBufWrite (io, "*CLS", 4, VI_NULL));
    viCheckErr( viReadSTB (io, &statusByte));

#else

    ViUInt16    response    = 0;
    ViInt32     counter     = 0;
    ViUInt16    interface   = VI_INTF_GPIB;
    ViInt32     opcTimeout = 0;
    //viCheckErr( viPrintf (io, "*OPC\n"));
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial interface */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
            {
#if 1
                error = viReadSTB (instrSession, &response);
#else
                error = viQueryf (instrSession, "*STB?\n","%ld", &response);
#endif

                Rs_Delay (0.001);
                counter++;

                if (counter >= opcTimeout)
                    return VI_ERROR_TMO;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and ESB bits are taken into consideration. */
            
            } while ((response & 0x24) == 0);
            /* (check Error(Event) Queue bit in STB) */
            return ((response & 0x4) != 0) ? RS_ERROR_INSTRUMENT_STATUS : error;
        }

#endif

Error:

#if 0

    viDiscardEvents (io, VI_EVENT_SERVICE_REQ, VI_QUEUE);

#endif

    return error;
}


